@@ -9,35 +9,32 @@
 //! at the index that the match starts at and its tree parent is
 //! resolved to the search element definition, we get a reference.
 
-use std::iter;
-
-use either::Either;
 use hir::{PathResolution, Semantics};
 use ide_db::{
     base_db::FileId,
     defs::{Definition, NameClass, NameRefClass},
-    search::{ReferenceAccess, SearchScope, UsageSearchResult},
+    search::{ReferenceCategory, SearchScope, UsageSearchResult},
     RootDatabase,
 };
 use rustc_hash::FxHashMap;
 use syntax::{
     algo::find_node_at_offset,
-    ast::{self, NameOwner},
+    ast::{self, HasName},
     match_ast, AstNode, SyntaxNode, TextRange, TextSize, T,
 };
 
-use crate::{display::TryToNav, FilePosition, NavigationTarget};
+use crate::{FilePosition, NavigationTarget, TryToNav};
 
 #[derive(Debug, Clone)]
 pub struct ReferenceSearchResult {
     pub declaration: Option<Declaration>,
-    pub references: FxHashMap<FileId, Vec<(TextRange, Option<ReferenceAccess>)>>,
+    pub references: FxHashMap<FileId, Vec<(TextRange, Option<ReferenceCategory>)>>,
 }
 
 #[derive(Debug, Clone)]
 pub struct Declaration {
     pub nav: NavigationTarget,
-    pub access: Option<ReferenceAccess>,
+    pub is_mut: bool,
 }
 
 // Feature: Find All References
@@ -58,57 +55,58 @@ pub(crate) fn find_all_refs(
 ) -> Option<Vec<ReferenceSearchResult>> {
     let _p = profile::span("find_all_refs");
     let syntax = sema.parse(position.file_id).syntax().clone();
+    let make_searcher = |literal_search: bool| {
+        move |def: Definition| {
+            let mut usages =
+                def.usages(sema).set_scope(search_scope.clone()).include_self_refs().all();
+            let declaration = match def {
+                Definition::Module(module) => {
+                    Some(NavigationTarget::from_module_to_decl(sema.db, module))
+                }
+                def => def.try_to_nav(sema.db),
+            }
+            .map(|nav| {
+                let decl_range = nav.focus_or_full_range();
+                Declaration {
+                    is_mut: decl_mutability(&def, sema.parse(nav.file_id).syntax(), decl_range),
+                    nav,
+                }
+            });
+            if literal_search {
+                retain_adt_literal_usages(&mut usages, def, sema);
+            }
 
-    let mut is_literal_search = false;
-    let defs = match name_for_constructor_search(&syntax, position) {
+            let references = usages
+                .into_iter()
+                .map(|(file_id, refs)| {
+                    (
+                        file_id,
+                        refs.into_iter()
+                            .map(|file_ref| (file_ref.range, file_ref.category))
+                            .collect(),
+                    )
+                })
+                .collect();
+
+            ReferenceSearchResult { declaration, references }
+        }
+    };
+
+    match name_for_constructor_search(&syntax, position) {
         Some(name) => {
-            is_literal_search = true;
             let def = match NameClass::classify(sema, &name)? {
                 NameClass::Definition(it) | NameClass::ConstReference(it) => it,
                 NameClass::PatFieldShorthand { local_def: _, field_ref } => {
                     Definition::Field(field_ref)
                 }
             };
-            Either::Left(iter::once(def))
+            Some(vec![make_searcher(true)(def)])
         }
-        None => Either::Right(find_defs(sema, &syntax, position.offset)),
-    };
-
-    Some(
-        defs.into_iter()
-            .map(|def| {
-                let mut usages =
-                    def.usages(sema).set_scope(search_scope.clone()).include_self_refs().all();
-                let declaration = match def {
-                    Definition::ModuleDef(hir::ModuleDef::Module(module)) => {
-                        Some(NavigationTarget::from_module_to_decl(sema.db, module))
-                    }
-                    def => def.try_to_nav(sema.db),
-                }
-                .map(|nav| {
-                    let decl_range = nav.focus_or_full_range();
-                    Declaration { nav, access: decl_access(&def, &syntax, decl_range) }
-                });
-                if is_literal_search {
-                    retain_adt_literal_usages(&mut usages, def, sema);
-                }
-
-                let references = usages
-                    .into_iter()
-                    .map(|(file_id, refs)| {
-                        (
-                            file_id,
-                            refs.into_iter()
-                                .map(|file_ref| (file_ref.range, file_ref.access))
-                                .collect(),
-                        )
-                    })
-                    .collect();
-
-                ReferenceSearchResult { declaration, references }
-            })
-            .collect(),
-    )
+        None => {
+            let search = make_searcher(false);
+            Some(find_defs(sema, &syntax, position.offset).into_iter().map(search).collect())
+        }
+    }
 }
 
 pub(crate) fn find_defs<'a>(
@@ -116,8 +114,8 @@ pub(crate) fn find_defs<'a>(
     syntax: &SyntaxNode,
     offset: TextSize,
 ) -> impl Iterator<Item = Definition> + 'a {
-    sema.find_nodes_at_offset_with_descend(syntax, offset).filter_map(move |node| {
-        Some(match node {
+    sema.find_nodes_at_offset_with_descend(syntax, offset).filter_map(move |name_like| {
+        let def = match name_like {
             ast::NameLike::NameRef(name_ref) => match NameRefClass::classify(sema, &name_ref)? {
                 NameRefClass::Definition(def) => def,
                 NameRefClass::FieldShorthand { local_ref, field_ref: _ } => {
@@ -138,31 +136,24 @@ pub(crate) fn find_defs<'a>(
                 .or_else(|| {
                     NameClass::classify_lifetime(sema, &lifetime).and_then(NameClass::defined)
                 })?,
-        })
+        };
+        Some(def)
     })
 }
 
-pub(crate) fn decl_access(
-    def: &Definition,
-    syntax: &SyntaxNode,
-    range: TextRange,
-) -> Option<ReferenceAccess> {
+pub(crate) fn decl_mutability(def: &Definition, syntax: &SyntaxNode, range: TextRange) -> bool {
     match def {
         Definition::Local(_) | Definition::Field(_) => {}
-        _ => return None,
+        _ => return false,
     };
 
-    let stmt = find_node_at_offset::<ast::LetStmt>(syntax, range.start())?;
-    if stmt.initializer().is_some() {
-        let pat = stmt.pat()?;
-        if let ast::Pat::IdentPat(it) = pat {
-            if it.mut_token().is_some() {
-                return Some(ReferenceAccess::Write);
-            }
-        }
+    match find_node_at_offset::<ast::LetStmt>(syntax, range.start()) {
+        Some(stmt) if stmt.initializer().is_some() => match stmt.pat() {
+            Some(ast::Pat::IdentPat(it)) => it.mut_token().is_some(),
+            _ => false,
+        },
+        _ => false,
     }
-
-    None
 }
 
 /// Filter out all non-literal usages for adt-defs
@@ -173,7 +164,7 @@ fn retain_adt_literal_usages(
 ) {
     let refs = usages.references.values_mut();
     match def {
-        Definition::ModuleDef(hir::ModuleDef::Adt(hir::Adt::Enum(enum_))) => {
+        Definition::Adt(hir::Adt::Enum(enum_)) => {
             refs.for_each(|it| {
                 it.retain(|reference| {
                     reference
@@ -184,7 +175,7 @@ fn retain_adt_literal_usages(
             });
             usages.references.retain(|_, it| !it.is_empty());
         }
-        Definition::ModuleDef(hir::ModuleDef::Adt(_) | hir::ModuleDef::Variant(_)) => {
+        Definition::Adt(_) | Definition::Variant(_) => {
             refs.for_each(|it| {
                 it.retain(|reference| reference.name.as_name_ref().map_or(false, is_lit_name_ref))
             });
@@ -280,7 +271,7 @@ fn is_lit_name_ref(name_ref: &ast::NameRef) -> bool {
 #[cfg(test)]
 mod tests {
     use expect_test::{expect, Expect};
-    use ide_db::base_db::FileId;
+    use ide_db::{base_db::FileId, search::ReferenceCategory};
     use stdx::format_to;
 
     use crate::{fixture, SearchScope};
@@ -765,7 +756,7 @@ use self$0;
             expect![[r#"
                 Module FileId(0) 0..10
 
-                (no references)
+                FileId(0) 4..8
             "#]],
         );
     }
@@ -1092,8 +1083,8 @@ impl Foo {
 
             if let Some(decl) = refs.declaration {
                 format_to!(actual, "{}", decl.nav.debug_render());
-                if let Some(access) = decl.access {
-                    format_to!(actual, " {:?}", access)
+                if decl.is_mut {
+                    format_to!(actual, " {:?}", ReferenceCategory::Write)
                 }
                 actual += "\n\n";
             }
@@ -1504,4 +1495,44 @@ fn f() {
             "#]],
         )
     }
+
+    #[test]
+    fn attr_expanded() {
+        check(
+            r#"
+//- proc_macros: identity
+
+#[proc_macros::identity]
+fn func$0() {
+    func();
+}
+"#,
+            expect![[r#"
+                func Function FileId(0) 26..51 29..33
+
+                FileId(0) 42..46
+            "#]],
+        )
+    }
+
+    #[test]
+    fn attr_assoc_item() {
+        check(
+            r#"
+//- proc_macros: identity
+
+trait Trait {
+    #[proc_macros::identity]
+    fn func() {
+        Self::func$0();
+    }
+}
+"#,
+            expect![[r#"
+                func Function FileId(0) 48..87 51..55
+
+                FileId(0) 74..78
+            "#]],
+        )
+    }
 }
